//---------------------------------------------------------------------------------------
/// \brief	This framework template sets up lua, fmod, cg, glew extensions. It also features
///			a frametimer and a file dialog....
///
//---------------------------------------------------------------------------------------

#ifndef __APP_FRAMEWORK__H__
#define __APP_FRAMEWORK__H__

	#ifdef WIN32
		#pragma once
		#pragma comment(lib,"LibApplication.lib")
	#endif

	//-----------------------------------------------------------------------------------
	//	Compilation flags
	//

	// The following flags determine which libs are linked into the APP

	// maya libraries
	#define USE_MLIBRARY 0

	// fmod audio
	#define USE_FMOD 1

	// CG runtime
	#define USE_CG 1

	// lua script
	#define USE_LUA 1

	// robs math lib
	#define USE_MATH 1

	// robs joypad lib
	#define USE_JOYPAD 1

	// set to 1 for debug output
	#define USE_DEBUG 0


	//-----------------------------------------------------------------------------------
	//	debugging Macros
	//

	// debugging macros
	#if USE_DEBUG
		#define ASSERT(X) \
			if( !(X) ) {											\
				std::cerr << "[ERROR] assertion failed,\n\tfile "	\
						<< __FILE__ << "\n\tline "				\
						<< __LINE__ << std::endl;					\
			}
		#define LOG(X) \
			std::cerr << __LINE__ << ": " << (X) << std::endl;
	#else
		#define ASSERT(X) ;
		#define LOG(X) ;
	#endif


	// standard C++ headers
	#include <cmath>
	#include <cstdlib>
	#include <iostream>
	#include <vector>
	#include <string>

	#if USE_LUA
		// lua headers
		extern "C" {
			#include <lua.h>
			#include <lualib.h>
			#include <lauxlib.h>
		}
		#include <luaBinder.h>
	#endif

	// openGL / glew / cg headers
	#include <GL/glew.h>
	#include <GL/glut.h>

	#if USE_CG
		#include <Cg/cg.h>
		#include <Cg/cgGL.h>
	#endif

	#if USE_FMOD
		#include <LibSound.h>
		#include <sound.h>
		#include <mp3.h>
	#endif

	#if USE_MATH
		// math lib headers
		#include <EulerAngles.h>
		#include <Interpolation.h>
		#include <Matrix.h>
		#include <Plane.h>
		#include <Quaternion.h>
		#include <Rand.h>
		#include <Transform.h>
		#include <Vector.h>
	#endif

	#if USE_MLIBRARY
		#include <maya/MLibrary.h>
		#include <maya/MString.h>
		#include <maya/MObject.h>
	#endif

	// useful application code
	#include <FrameTimer.h>
	#include <FileDialog.h>
	#include <Font.h>
	#include <Joypad.h>

	// redefinitions of the key values. All of the keys are written into a single array
	// for the Polled keyboard IO. The Keypress overide functions will recieve these values
	// rather than the glut key codes (normal keys remain unchanged)
	//
	#define  KEY_F1 		(256+GLUT_KEY_F1)
	#define  KEY_F2 		(256+GLUT_KEY_F2)
	#define  KEY_F3 		(256+GLUT_KEY_F3)
	#define  KEY_F4 		(256+GLUT_KEY_F4)
	#define  KEY_F5 		(256+GLUT_KEY_F5)
	#define  KEY_F6 		(256+GLUT_KEY_F6)
	#define  KEY_F7 		(256+GLUT_KEY_F7)
	#define  KEY_F8 		(256+GLUT_KEY_F8)
	#define  KEY_F9 		(256+GLUT_KEY_F9)
	#define  KEY_F10 		(256+GLUT_KEY_F10)
	#define  KEY_F11 		(256+GLUT_KEY_F11)
	#define  KEY_F12 		(256+GLUT_KEY_F12)
	#define  KEY_LEFT 		(256+GLUT_KEY_LEFT)
	#define  KEY_UP 		(256+GLUT_KEY_UP)
	#define  KEY_RIGHT 		(256+GLUT_KEY_RIGHT)
	#define  KEY_DOWN 		(256+GLUT_KEY_DOWN)
	#define  KEY_PAGE_UP 	(256+GLUT_KEY_PAGE_UP)
	#define  KEY_PAGE_DOWN 	(256+GLUT_KEY_PAGE_DOWN)
	#define  KEY_HOME 		(256+GLUT_KEY_HOME)
	#define  KEY_END 		(256+GLUT_KEY_END)
	#define  KEY_INSERT 	(256+GLUT_KEY_INSERT)

	// some handy key values
	#define  KEY_ENTER		13
	#define  KEY_TAB		9
	#define  KEY_ESCAPE		27
	#define  KEY_BACKSPACE	8
	#define  KEY_DELETE		127
	#define  KEY_CONSOLE	96

	//-----------------------------------------------------------------------------------
	///
	///
	///
	class XApplication
	{
	public:

		/// \brief	returns true if a key is pressed
		/// \return	true if held
		bool IsKeyPressed(int key) const;

		/// \brief	This function allows us to execute lua code stored in a text string
		/// \param	text	-	the lua code
		/// \return	true if OK
		bool ExecuteString(const char*);

		/// \brief	This function allows us to execute lua code stored in a file
		/// \param	file	-	the filename of the lua script
		/// \return	true if OK
		bool ExecuteFile(const char*);

		/// \brief	Displays a fairly rubbish console that can enter pretty basic
		/// 		lua commands.
		/// \param
		void EnableConsole(bool);


	public:

		/// \brief	Overload this function to do custom app initialisation
		virtual void OnInit() ;

		/// \brief	Overload this function to register your own script functions
		virtual void OnRegisterFunctions() ;

		/// \brief	Overload this function to delete any resources or dynamic data
		virtual void OnExit() ;


		/// \brief	Overload this function to update your application
		/// \param	dt	-	the time elapsed since the last frame
		virtual void OnUpdate(float dt) ;


		/// \brief	by default this calls glOrtho(0,w,0,h,-100,100);
		/// 		overload it to set your own 2D projection
		/// \param	w	-	the new window width
		/// \param	h	-	the new window height
		virtual void OnReshape2D(int w,int h);

		/// \brief	by default this calls gluPerspective(45,(float)w/h,0.1,100);
		/// 		overload it to set your own 3D projection
		/// \param	w	-	the new window width
		/// \param	h	-	the new window height
		virtual void OnReshape3D(int w,int h);



		/// \brief	Overload this function to draw any 2D stuff
		virtual void OnDraw2D() ;

		/// \brief	Overload this function to draw any 3D stuff
		virtual void OnDraw3D() ;



	public:

		/// \brief	overload this to handle mouse button presses and releases
		/// \param	button	-	GLUT_LEFT_BUTTON,GLUT_MIDDLE_BUTTON or GLUT_RIGHT_BUTTON
		/// \param	state	-	GLUT_DOWN or GLUT_UP
		/// \param	x		-	the x coord of the mouse
		/// \param	y		-	the y coord of the mouse
		virtual void OnMouseButton(int button, int state,int x,int y);

		/// \brief	overload this to handle the movement of the mouse when a button is held
		/// \param	x		-	the x coord of the mouse
		/// \param	y		-	the y coord of the mouse
		virtual void OnMouseMove(int x,int y);

		/// \brief	overload this to handle the passive motion of the mouse
		/// \param	x		-	the x coord of the mouse
		/// \param	y		-	the y coord of the mouse
		virtual void OnMousePassive(int x,int y);


		/// \brief	Overload this function to handle a key press event
		/// \param	key		-	the key code, either ascii value or one of defines above
		/// \param	x		-	the x coord of the mouse
		/// \param	y		-	the y coord of the mouse
		virtual void OnKeyPress(int,int,int);

		/// \brief	Overload this function to handle a key release event
		/// \param	key		-	the key code, either ascii value or one of defines above
		/// \param	x		-	the x coord of the mouse
		/// \param	y		-	the y coord of the mouse
		virtual void OnKeyRelease(int,int,int);

		/// ctor
		XApplication() {};

		/// dtor
		virtual ~XApplication() {};

	};

	extern XApplication* CreateApp();

#if USE_CG

	extern void BindPrograms(CGprogram vert, CGprogram frag);

	extern void UnBindPrograms();

	/// the CG context
	extern const CGcontext& GetCGContext() ;

	/// the CG vertex profile used
	extern const CGprofile& GetVertexProfile() ;

	/// the CG fragment profile used
	extern const CGprofile& GetFragmentProfile() ;

#endif


#endif













